/*
******************************************************************************

   Fallback stubs for BLT implementation - transfer batchop.

   Copyright (C) 2001	Brian S. Julin [bri@tull.umassp.edu]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include "stubsblt.h"

/* Fields in here are mostly ignored, since we know the row format
 * and implement our own go.  The batchparms are really only used
 * to match up fields from user-generated batchops.
 */
struct batchparm BLT_stubs_bp[BLT_STUBS_NUMPARMS] = {
#define BLT_STUBS_ROW_GET1	0
  { parmtype: GGI_PT_IS_OPCODE,		/* row.get1 */
  },
#define BLT_STUBS_ROW_GET2	1
  { parmtype: GGI_PT_IS_OPCODE,		/* row.get2 */
  },
#define BLT_STUBS_ROW_GET1X	2
  { parmtype: BLT_PT_SRCX,		/* row.get1x */
  },
#define BLT_STUBS_ROW_GET1Y	3
  { parmtype: BLT_PT_SRCY,		/* row.get1y */
  },
#define BLT_STUBS_ROW_GET1BUF	4
  { parmtype: BLT_PT_SRCADDR,		/* row.get1buf */
  },
#define BLT_STUBS_ROW_PUT1	5
  { parmtype: GGI_PT_IS_OPCODE,		/* row.put1 */
  },
#define BLT_STUBS_ROW_PUT2	6
  { parmtype: GGI_PT_IS_OPCODE,		/* row.put2 */
  },
#define BLT_STUBS_ROW_PUT1X	7
  { parmtype: BLT_PT_DSTX,		/* row.put1x */
  },
#define BLT_STUBS_ROW_PUT1Y	8
  { parmtype: BLT_PT_DSTY,		/* row.put1y */
  },
#define BLT_STUBS_ROW_PUT1BUF	9
  { parmtype: BLT_PT_DSTADDR,		/* row.put1buf */
  },
};

static int BLT_stubs_pt_callback (int srcpt, int dstpt) {
	/* This is a cop-out for now -- we only serve exact matches */
  fprintf(stderr,"trying %x vs %x\n", srcpt, dstpt);
	if (srcpt != dstpt) return 0;
	return 1;
}	/* _bo_def_pt_callback */

static inline int BLT_stubs_match_parm(batchop_t s, batchparm_t p, 
				       bo_pt_callback *cmp) {
	int idx = -1;
	batchparm_t cur;

	p->match.src = -1;

	if (cmp == NULL) cmp = BLT_stubs_pt_callback;

	while (++idx < s->numparms) {
		batchparm_t try;
		int try_res;

		try = &(s->parms[idx]);

		try_res = cmp(try->parmtype, p->parmtype);
                if (!try_res) continue;
		if (p->match.src == -1) {
			p->match.src = idx;
			continue;
                }
        }
	cur = &(s->parms[p->match.src]);
	return BO_OK;
}


int BLT_stubs_bo_match (batchop_t source, batchop_t dest, bo_pt_callback *cmp) 
{
	int idx;
	for (idx = 0; idx < dest->numparms; idx++)
		if (BLT_stubs_match_parm(source, &(dest->parms[idx]), cmp)) 
			return -1;

	dest->lastmatch = source;

	dest->go = BLT_stubs_bo_go;

	return BO_OK;
}	/* bo_def_match */

int BLT_stubs_bo_go (batchop_t source, batchop_t dest, int numrows) {
	return 0;
}
