/*
******************************************************************************

   BLT debugging macros

   Copyright (C) 2001 Christoph Egger	[Christoph_Egger@t-online.de]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _BLT_INTERNAL_DEBUG_H
#define _BLT_INTERNAL_DEBUG_H


#include <ggi/internal/blt.h>

#include <stdio.h>
#include <stdarg.h>
#include <ggi/types.h>
#include <ggi/gg.h>



__BEGIN_DECLS

/* Exported variables */
#ifdef BUILDING_LIBBLT
extern uint32     _bltDebugState;
extern int        _bltDebugSync;
#else
IMPORTVAR uint32  _bltDebugState;
IMPORTVAR int     _bltDebugSync;
#endif
__END_DECLS


/* Debugging types
 * bit 0 is reserved! */

#define BLTDEBUG_CORE		(1<<1)	/*   2 */
#define BLTDEBUG_MODE		(1<<2)	/*   4 */
#define BLTDEBUG_COLOR		(1<<3)	/*   8 */
#define BLTDEBUG_DRAW		(1<<4)	/*  16 */
#define BLTDEBUG_MISC		(1<<5)	/*  32 */
#define BLTDEBUG_LIBS		(1<<6)	/*  64 */
#define BLTDEBUG_EVENTS		(1<<7)	/* 128 */

#define BLTDEBUG_ALL	0xffffffff

#define DEBUG_INFO	__FILE__,__PRETTY_FUNCTION__,__LINE__

#ifdef DEBUG
#define LIB_ASSERT(x)	\
	if (!(x))	\
		fprintf(stderr,"LIBBLT:%s:%s:%d: assertion \"%s\" failed.\n", DEBUG_INFO, #x);
#else
#define LIB_ASSERT(x)	do{}while(0)
#endif


#ifdef __GNUC__

#ifdef DEBUG
#define BLTDPRINT(args...)	   if (_bltDebugState) { ggDPrintf(_bltDebugSync, "LibBLT",args); }
#define BLTDPRINT_CORE(args...)   if (_bltDebugState & BLTDEBUG_CORE) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#define BLTDPRINT_MODE(args...)   if (_bltDebugState & BLTDEBUG_MODE) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#define BLTDPRINT_COLOR(args...)  if (_bltDebugState & BLTDEBUG_COLOR) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#define BLTDPRINT_DRAW(args...)   if (_bltDebugState & BLTDEBUG_DRAW) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#define BLTDPRINT_MISC(args...)   if (_bltDebugState & BLTDEBUG_MISC) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#define BLTDPRINT_LIBS(args...)   if (_bltDebugState & BLTDEBUG_LIBS) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#define BLTDPRINT_EVENTS(args...) if (_bltDebugState & BLTDEBUG_EVENTS) { ggDPrintf(_bltDebugSync,"LibBLT",args); }
#else /* DEBUG */
#define BLTDPRINT(args...)		do{}while(0)
#define BLTDPRINT_CORE(args...)	do{}while(0)
#define BLTDPRINT_MODE(args...)	do{}while(0)
#define BLTDPRINT_COLOR(args...)	do{}while(0)
#define BLTDPRINT_DRAW(args...)	do{}while(0)
#define BLTDPRINT_MISC(args...)	do{}while(0)
#define BLTDPRINT_LIBS(args...)	do{}while(0)
#define BLTDPRINT_EVENTS(args...)	do{}while(0)
#endif /* DEBUG */

#else /* __GNUC__ */

__BEGIN_DECLS

static inline void BLTDPRINT(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_CORE(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_CORE) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_MODE(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_MODE) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_COLOR(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_COLOR) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_DRAW(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_DRAW) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_MISC(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_MISC) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_LIBS(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_LIBS) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

static inline void BLTDPRINT_EVENTS(const char *form,...)
{
#ifdef DEBUG
	if (_bltDebugState & BLTDEBUG_EVENTS) {
		va_list args;

		fprintf(stderr, "BLT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_bltDebugSync) fflush(stderr);
	}
#endif
}

__END_DECLS

#endif /* __GNUC__ */

#ifdef DEBUG
#define BLT_ASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"BLT:%s:%d: INTERNAL ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#define BLT_APPASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"BLT:%s:%d: APPLICATION ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#else /* DEBUG */
#define BLT_ASSERT(x,str)	do{}while(0)
#define BLT_APPASSERT(x,str)	do{}while(0)
#endif /* DEBUG */

#ifdef DEBUG
# define BLTD0(x)	x
#else
# define BLTD0(x)	/* empty */
#endif

#ifdef BLTDLEV
# if BLTDLEV == 1
#  define BLTD1(x)	x
#  define BLTD2(x)	/* empty */
#  define BLTD3(x)	/* empty */
# elif BLTDLEV == 2
#  define BLTD1(x)	x
#  define BLTD2(x)	x
#  define BLTD3(x)	/* empty */
# elif BLTDLEV > 2
#  define BLTD1(x)	x
#  define BLTD2(x)	x
#  define BLTD3(x)	x
# endif
#else
# define BLTD1(x)	/* empty */
# define BLTD2(x)	/* empty */
# define BLTD3(x)	/* empty */
#endif

#endif /* _BLT_INTERNAL_DEBUG_H */
