/*
******************************************************************************

   LibOVL: extension API header file

   Copyright (C) 2001 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#ifndef _GGI_OVL_H
#define _GGI_OVL_H

#ifndef _GGI_INTERNAL_GII_H
#include <ggi/ggi.h>
#endif

__BEGIN_DECLS

#include <ggi/galloc.h>

/* Extension management
 */
int ggiOvlInit(void);
int ggiOvlExit(void);

int ggiOvlAttach(ggi_visual_t vis, ggiGA_resource_list *reqlist);
int ggiOvlDetach(ggi_visual_t vis);


/* Structure
 */

#ifdef HAVE_STRUCT_OVL
typedef struct ggiOvl *ggiOvl_t;
#else
struct ggiOvl { };

typedef struct ggiOvl *ggiOvl_t;
#endif


/* Flags
 */

enum ggiOvl_flags {

	/* autotrack-feature */
	OVL_FLAG_AUTOTRACK		= GA_FLAG_AUTOTRACK,

	/* priority flag */
	OVL_FLAG_ALWAYS_ON_TOP		= GA_FLAG_ALWAYS_ON_TOP,


	/* Video specific
	 */

	/* video formats */
	OVL_FLAG_FORMAT_MPEG_1		= GA_FLAG_VIDEO_MPEG_1,
	OVL_FLAG_FORMAT_MPEG_2		= GA_FLAG_VIDEO_MPEG_2,
	OVL_FLAG_FORMAT_MPEG_4		= GA_FLAG_VIDEO_MPEG_4,
	OVL_FLAG_FORMAT_AVI		= GA_FLAG_VIDEO_AVI,
	OVL_FLAG_FORMAT_QUICKTIME	= GA_FLAG_VIDEO_QUICKTIME,
	OVL_FLAG_FORMAT_H261		= GA_FLAG_VIDEO_H261,
	OVL_FLAG_FORMAT_H263		= GA_FLAG_VIDEO_H263,

	/* operations during display */
	OVL_FLAG_INTERPOLATE		= GA_FLAG_VIDEO_INTERPOLATE,
	OVL_FLAG_DECOMPRESS		= GA_FLAG_VIDEO_DECOMPRESS,

	/* framerate */
	OVL_FLAG_CURRENT_FRAMERATE	= GA_FLAG_VIDEO_CURFRAMERATE,
	OVL_FLAG_AVERAGE_FRAMERATE	= GA_FLAG_VIDEO_AVGFRAMERATE,
};



/* API functions
 */

/* type is always a subtype of either
 * GA_RT_SPRITE_*, GA_RT_VIDEO_* or GA_RT_WINDOW_*.
 * Otherwise, everything fails.
 */

/* On-the-fly beginners interface */

ggiOvl_t ggiOvlCreateSprite(ggi_visual_t vis,
			    long width, long height,
			    enum ggiGA_resource_type subtype,
			    enum ggiOvl_flags flags);

ggiOvl_t ggiOvlCreateWindow(ggi_visual_t vis,
			    long width, long height,
			    enum ggiGA_resource_type subtype,
			    enum ggiOvl_flags flags);

ggiOvl_t ggiOvlCreateVideo(ggi_visual_t vis,
			   long width, long height,
			   enum ggiGA_resource_type subtype,
			   enum ggiOvl_flags flags);

int ggiOvlClearMotorProperties (struct ggiGA_resource_props *prop);
int ggiOvlClearCarbProperties  (struct ggiGA_resource_props *prop);
int ggiOvlClearTankProperties  (struct ggiGA_resource_props *prop);

ggiOvl_t ggiOvlCreate(ggi_visual_t vis,
		      struct ggiGA_resource_props *prop_motor,
		      struct ggiGA_resource_props *prop_carb,
		      struct ggiGA_resource_props *prop_tank,
		      enum ggiGA_resource_type type);

ggiOvl_t ggiOvlCreateInGroup(ggi_visual_t vis, ggiOvl_t group_leader,
			     struct ggiGA_resource_props *prop_tank);

int ggiOvlDestroyOvl(ggiOvl_t *ovl);


/* Interface for the LibGAlloc adept. */

int ggiOvlAdd(ggiGA_resource_list *reqlist,
	      struct ggiGA_resource_props *prop_motor,
	      struct ggiGA_resource_props *prop_carb,
	      struct ggiGA_resource_props *prop_tank,
	      struct ggiGA_resource_props *prop_tank_cap,
	      enum ggiGA_resource_type type,
	      ggiGA_resource_handle *handle_motor,
	      ggiGA_resource_handle *handle_carb,
	      ggiGA_resource_handle *handle_tank,
	      ggiGA_resource_handle *handle_tank_cap);

int ggiOvlAdd2Group(ggiGA_resource_list *reqlist,
		    struct ggiGA_resource_props *prop_tank,
		    struct ggiGA_resource_props *prop_tank_cap,
		    ggiGA_resource_handle handle_carb,
		    ggiGA_resource_handle *handle_tank,
		    ggiGA_resource_handle *handle_tank_cap);

ggiOvl_t ggiOvlTie(ggi_visual_t vis,
		   ggiGA_resource_handle handle_motor,
		   ggiGA_resource_handle handle_carb,
		   ggiGA_resource_handle handle_tank);

int ggiOvlUntie(ggiOvl_t *ovl);

ggiGA_resource_handle ggiOvlGetMotorHandle	(ggiOvl_t ovl);
ggiGA_resource_handle ggiOvlGetCarbHandle	(ggiOvl_t ovl);
ggiGA_resource_handle ggiOvlGetTankHandle	(ggiOvl_t ovl);

int ggiOvlHide(ggiOvl_t ovl);
int ggiOvlIsHidden(ggiOvl_t ovl);
int ggiOvlShow(ggiOvl_t ovl);
int ggiOvlSetPriority(ggiOvl_t ovl, uint32 slot, int swap);
int ggiOvlGetPriority(ggiOvl_t ovl, uint32 *slot);

int ggiOvlSetPos(ggiOvl_t ovl, long x, long y);
int ggiOvlMove(ggiOvl_t ovl, long x, long y);

int ggiOvlEnable(ggiOvl_t ovl);
int ggiOvlDisable(ggiOvl_t ovl);
int ggiOvlIsEnabled(ggiOvl_t ovl);

int ggiOvlGetPos(ggiOvl_t ovl, long *x, long *y);
int ggiOvlSetSize(ggiOvl_t ovl, long w, long h);
int ggiOvlGetSize(ggiOvl_t ovl, long *x, long *y);
int ggiOvlSetZoom(ggiOvl_t ovl, long x, long y, long w, long h, int keep_ratio);
int ggiOvlGetZoom(ggiOvl_t ovl, long *x, long *y, long *w, long *h);
int ggiOvlSetVals(ggiOvl_t ovl, long x, long y, long w, long h,
		  long zx, long zy, long zw, long zh);

int ggiOvlSetCoordBase(ggiOvl_t ovl, unsigned int flags);

int ggiOvlSetFlags(ggiOvl_t, unsigned int flags);

/* Set the given image (vis) to the ovl-area with grabbing the image
 * from a visual at pos (x,y)
 */
int ggiOvlSetImage(ggiOvl_t ovl, ggi_visual_t vis,
		   long x, long y, unsigned char *alpha);

__END_DECLS

#endif /* _GGI_OVL_H */


